#! /usr/bin/env bash

set -e

# Note: this line is edited at compile time to insert the install folder
PREFIX=\

usage_string=$(cat << EOF
usage: fxsdk new <folder>
       fxsdk (build|build-fx|build-cg) [-s]
       fxsdk (send|send-fx|send-cg)

This program is a command-line helper for the fxSDK, a set of tools used in
conjunction with gint to develop add-ins for CASIO fx-9860G and fx-CG 50.

Project creation:
  fxsdk new <folder>

  Creates a new project in the specified folder. Project info is input
  interactively. Creates <folder name> and populates it with default project
  data.

Compilation:
  fxsdk build [-s]
  fxsdk build-fx [-s]
  fxsdk build-cg [-s]

  Compiles the current project for fx-9860G (.g1a target) or fx-CG 50 (.g3a
  target). With 'fxsdk build', compiles every existing build folder, and ask
  interactively if none is found.

  With '-s', also sends the resulting program to the calculator.

Installation:
  fxsdk send
  fxsdk send-fx
  fxsdk send-cg

  Sends the target file to the calculator. Uses p7 (which must be installed
  externally) for fx-9860G. Currently not implemented for fx-CG 50, as it
  requires detecting and mounting the calculator (same for the Graph 35+E II).

Project update:
  fxsdk update

  Copies the latest version of the Makefile to your project. *This will
  discard any changes made to your Makefile.* If you have edited your
  Makefile, make a backup and merge the changes after updating.
EOF
)

usage() {
  echo "$usage_string"
  exit ${1:-1}
}

error() {
  echo -n "error: " >&2
  echo "$@" >&2
  echo "Try 'fxsdk --help' for more information." >&2
}

status() {
    echo -ne "\n\e[34;1m::\e[39;1m "
    echo -n "$@"
    echo -e "\e[0m\n"
}

fxsdk_new_project_interactive() {
  [[ -z "$1" ]] && error "please specify the project folder" && return 1
  [[ -e "$1" ]] && error "'$1' exists, I don't dare touch it" && return 1

  echo -e "Creating a new project in folder '$1'.\n"

  echo -ne "Full project name ? (at most 8 characters)\n> "
  read NAME

  echo "Internal name ? ('@' followed by at most 7 uppercase letters)"
  echo -ne "(Add-in might not appear on calc if format is wrong)\n> "
  read INTERNAL

  mkdir -p "$1"/{,src,assets-fx,assets-cg}

  assets="$PREFIX/share/fxsdk/assets"

  sed -e "s/@NAME@/$NAME/g" -e "s/@INTERNAL@/$INTERNAL/g" \
    "$assets/project.cfg" > "$1/project.cfg"

  cp "$assets"/Makefile "$1"
  cp "$assets"/gitignore "$1"/.gitignore
  cp "$assets"/main.c "$1"/src
  cp "$assets"/icon-fx.png "$1"/assets-fx
  cp "$assets"/icon-cg-uns.png "$1"/assets-cg
  cp "$assets"/icon-cg-sel.png "$1"/assets-cg

  echo -e "\nYour project '$NAME' has been created.\n"
  echo "Type 'fxsdk build-fx' or 'fxsdk build-cg' to compile the program."
}

fxsdk_load_config() {
  grep -E '^ *[a-zA-Z0-9_]+ *=' project.cfg \
  | sed -E 's/^([A-Z_]+)\s*=\s*(.*)/\1="\2"/' \
  | source /dev/stdin
}


fxsdk_build() {
  built=

  if [[ -e build-fx ]]; then
    status "Making into build-fx"
    make all-fx
    built=1
  fi

  if [[ -e build-cg ]]; then
    status "Making into build-cg"
    make all-cg
    built=1
  fi

  [[ $built ]] && return

  echo "No build files currently exist. Which platform do I compile for?"
  echo ""
  echo '"fx": fx-9860G II, Graph 35+ USB/E/E II, Graph 75+'
  echo '"cg": fx-CG 10/20/50, Graph 90+E'
  echo ""
  echo "Leave blank to cancel."

  platform=
  while true; do
    echo -n '> '
    read platform
    [[ -z $platform ]] && return

    if [[ $platform == "fx" ]]; then
      status "Making into build-fx"
      make all-fx
      return
    fi

    if [[ $platform == "cg" ]]; then
      status "Making into build-fx"
      make all-cg
      return
    fi

    echo -e 'Unknown platform (valid names are "fx" and "cg")!'
  done
}

fxsdk_build_fx() {
  status "Making into build-fx"
  make all-fx
}

fxsdk_build_cg() {
  status "Making into build-cg"
  make all-cg
}

fxsdk_send() {
  if [[ -e "build-fx" && ! -e "build-cg" ]]; then
    fxsdk_send_fx
  fi

  if [[ -e "build-cg" && ! -e "build-fx" ]]; then
    fxsdk_send_cg
  fi

  echo "either no or several platforms are targeted, use 'fxsdk send-fx' or"
  echo "fxsdk 'send-cg' to specify which calculator to send to."
}

fxsdk_send_fx() {
  status "Installing for fx9860g using p7"
  make install-fx
}

fxsdk_send_cg() {
  # TODO
  echo "error: this is tricky and not implemented yet, sorry x_x"
}

fxsdk_update() {
  if [[ ! -e "project.cfg" ]]; then
    echo "No file 'project.cfg' was found. This does not look like an fxSDK"
    echo "project folder. (Nothing done.)"
  fi

  assets="$PREFIX/share/fxsdk/assets"
  cp "$assets"/Makefile .
  echo "Succesfully copied $assets/Makefile to the current directory."
}

# Parse command name

case $1 in

  # Project creation
  "new")
    shift
    fxsdk_new_project_interactive "$@";;

  # Project compilation
  "build"|"b")
    fxsdk_build
    if [[ "$2" == "-s" ]]; then
      fxsdk_send
    fi;;
  "build-fx"|"bf"|"bfx")
    fxsdk_build_fx
    if [[ "$2" == "-s" ]]; then
      fxsdk_send_fx
    fi;;
  "build-cg"|"bc"|"bcg")
    fxsdk_build_cg
    if [[ "$2" == "-s" ]]; then
      fxsdk_send_cg
    fi;;

  # Install
  "send"|"s")
    fxsdk_send;;
  "send-fx"|"sf"|"sfx")
    fxsdk_send_fx;;
  "send-cg"|"sc"|"scg")
    fxsdk_send_cg;;

  # Project update
  "update")
    fxsdk_update;;

  # Misc
  -h|--help|-\?)
    usage 0;;
  ?*)
    error "unknown command '$1'"
    exit 1;;
  *)
    usage 0;;
esac
